import torch
from torch import nn
from zeta import (
    AutoregressiveWrapper,
    Decoder,
    Encoder,
    Transformer,
    ViTransformerWrapper,
)

class Kosmos(nn.Module):
    def __init__(
        self,
        image_size = 256,
        patch_size = 32,
        encoder_dim = 512,
        encoder_depth = 6,
        encoder_heads=8,
        num_tokens = 108481,
        max_seq_len = 8192,
        decoder_dim = 2560,
        decoder_depth = 32,
        decoder_heads = 24,
        use_abs_pos_emb = False,
        alibi_pos_bias=True,
        alibi_num_heads=12,
        rotary_xpos=True,
        attn_flash=True,
        qk_norm=True,
    ):
        super(Kosmos, self).__init__()

        self.encoder = ViTransformerWrapper(
            image_size = image_size,
            patch_size = patch_size,
            attn_layers = Encoder(
                dim=encoder_dim,
                depth=encoder_depth,
                heads=encoder_heads
            )
        )

        self.decoder = Transformer(
            num_token=num_tokens,
            max_seq_len=max_seq_len,
            use_abs_pos_emb=use_abs_pos_emb,
            attn_layers=Decoder(
                dim=decoder_dim,
                depth=decoder_depth,
                heads=decoder_heads,
                cross_attend=True,
                alibi_pos_bias=alibi_pos_bias,
                alibi_num_heads=alibi_num_heads,
                rotary_xpos=rotary_xpos,
                attn_flash=attn_flash,
                qk_norm=qk_norm,
            )
        )

        self.decoder = AutoregressiveWrapper(self.decoder)
    
    def forward(self, img, text):
        try:
            encoded = self.encoder(img, return_embeddings=True)
            return self.decoder(text, context=encoded)
        except Exception as error:
            print(f"Failed in forwrd method: {error}")
            raise
        


import os
from logging import getLogger
from typing import List

from sentencepiece import SentencePieceProcessor

logger = getLogger(__name__)

class Tokenizer:
    def __init__(
        self,
        model_path: str
    ):
        assert os.path.isfile(model_path), model_path

        self.sp_model = SentencePieceProcessor(model_path=model_path)
        logger.info(f"Reloaded SentiencePiece model from {model_path}")

        # BOS / EOS token IDs
        self.n_words: int = self.sp_model.vocab_size()
        self.bos_id: int = self.sp_model.bos_id()
        self.eos_id: int = self.sp_model.eos_id()
        self.pad_id: int = self.sp_model.pad_id()
        
        self.img_token = "<img>"
        self.img_token_close = "</img>"
        self.img_token_id = self.sp_model.piece_to_id(self.img_token)
        self.img_token_close_id = self.sp_model.piece_to_id(self.img_token_close)



        logger.info(
            f"#Words: {self.n_words}, - BOS ID: {self.bos_id} - EOS ID: {self.eos_id}"
        )
        assert self.sp_model.vocab_size() == self.sp_model.get_piece_size()

    def encode(
        self,
        s: str,
        bos: bool,
        eos: bool
    ) -> List[int]:
        assert type(s) is str
        t = self.sp_model.encode(s)
        
        if bos:
            t = [self.bos_id] + t
        if eos:
            t = t + [self.eos_id]
        return t
    
    def decode(
        self,
        t: List[int]
    ) -> str:
        return self.sp_model.decode(t)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kosmos']

package_data = \
{'': ['*']}

install_requires = \
['torch', 'zeta']

setup_kwargs = {
    'name': 'kosmos2-torch',
    'version': '0.0.1',
    'description': 'Kosmos - Pytorch',
    'long_description': '[![Multi-Modality](agorabanner.png)](https://discord.gg/qUtxnK2NMf)\n\n# Kosmos2.5\nMy implementation of Kosmos2.5 from Microsoft research and the paper: "KOSMOS-2.5: A Multimodal Literate Model"\n\n[Paper Link](https://arxiv.org/pdf/2309.11419.pdf)\n\n# Appreciation\n* Lucidrains\n* Agorians\n\n\n\n# Install\n\n\n# Dataset Strategy\nHere is a table summarizing the datasets used in the paper KOSMOS-2.5: A Multimodal Literate Model with metadata and source links:\n\n| Dataset | Modality | # Samples | Domain | Source | \n|-|-|-|-|-|  \n| IIT-CDIP | Text + Layout | 27.6M pages | Scanned documents | [Link](https://ir.nist.gov/cdip/)|\n| arXiv papers | Text + Layout | 20.9M pages | Research papers | [Link](https://arxiv.org/) |  \n| PowerPoint slides | Text + Layout | 6.2M pages | Presentation slides | Web crawl |\n| General PDF | Text + Layout | 155.2M pages | Diverse PDF files | Web crawl |\n| Web screenshots | Text + Layout | 100M pages | Webpage screenshots | [Link](https://www.tensorflow.org/datasets/catalog/c4) |\n| README | Text + Markdown | 2.9M files | GitHub README files | [Link](https://github.com/) |  \n| DOCX | Text + Markdown | 1.1M pages | WORD documents | Web crawl |\n| LaTeX | Text + Markdown | 3.7M pages | Research papers | [Link](https://arxiv.org/) |\n| HTML | Text + Markdown | 6.3M pages | Webpages | [Link](https://www.tensorflow.org/datasets/catalog/c4) |\n\n\n\n# License\nMIT\n\n# Citations\n```bibtex\n@misc{2309.11419,\nAuthor = {Tengchao Lv and Yupan Huang and Jingye Chen and Lei Cui and Shuming Ma and Yaoyao Chang and Shaohan Huang and Wenhui Wang and Li Dong and Weiyao Luo and Shaoxiang Wu and Guoxin Wang and Cha Zhang and Furu Wei},\nTitle = {Kosmos-2.5: A Multimodal Literate Model},\nYear = {2023},\nEprint = {arXiv:2309.11419},\n}\n```\n\n**bold**\n*italics*\n',
    'author': 'Kye Gomez',
    'author_email': 'kye@apac.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kyegomez/Kosmos2.5',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
